﻿local L = AceLibrary("AceLocale-2.2"):new("GridStatusMissingBuffs")
local Aura = AceLibrary("SpecialEvents-Aura-2.0")
local BS = AceLibrary("Babble-Spell-2.2")
local BC = AceLibrary("Babble-Class-2.2")
local RL = AceLibrary("Roster-2.1")

L:RegisterTranslations("enUS", function()
	return {
		["Missing Buffs"] = true,
		["Buff Group: "] = true,
		["[^%a]"] = true,
		["Show on %s."] = true,

		["Add Buff"] = true,
		["Adds a new buff to the buff group"] = true,
		["<buff name>"] = true,
		["Delete Buff"] = true,
		["Deletes an existing buff from the buff group"] = true,
		["Buff Icon"] = true,
		["The icon to appear when no buffs in the buff group are active."] = true,
		["<setName>"] = true,
		["Show"] = true,
		["Configure when to show the buff group"] = true,
			["When Dead"] = true,
			["Show missing buff group for dead players"] = true,
			["When Offline"] = true,
			["Show missing buff group for offline players"] = true,
			["In Combat"] = true,			
			["Show missing buff group when in combat"] = true,
		["Class Filter"] = true,
		["Show status for the selected classes."] = true,

		["Add Buff Group"] = true,
		["Adds a new buff group to the status module"] = true,
		["Delete Buff Group"] = true,
		["Deletes an existing buff group from the status module"] = true,
		["Remove %s from the menu"] = true,
	}
end)

L:RegisterTranslations("koKR", function()
	return {
		["Missing Buffs"] = "버프 사라짐",
		["Buff Group: "] = "그룹 버프: ",
		["[^%a]"] = "[^%S]",
		["Show on %s."] = "%s 표시",

		["Add Buff"] = "버프 추가",
		["Adds a new buff to the buff group"] = "그룹 버프에 새로운 버프를 추가합니다.",
		["<buff name>"] = "<버프 이름>",		
		["Delete Buff"] = "버프 삭제",
		["Deletes an existing buff from the buff group"] = "그룹 버프에 버프를 삭제합니다." ,
		["Buff Icon"] = "버프 아이콘",
		["The icon to appear when no buffs in the buff group are active."] = "그룹 버프의 버프가 없을때 아이콘을 표시합니다.",
		["<setName>"] = "<세트이름>", 
		["Show"] = "표시", 
		["Configure when to show the buff group"] = "그룹 버프 표시 설정을 합니다.",
			["When Dead"] = "죽었을때",
			["Show missing buff group for dead players"] = "죽은 플레이어를 위해 그룹 버프가 없을때 표시합니다.",
			["When Offline"] = "오프라인일때",
			["Show missing buff group for offline players"] = "오프라인 플레이어를 위해 그룹 버프가 없을때 표시합니다.",
			["In Combat"] = "전투상태일때",	
			["Show missing buff group when in combat"] = "전투중 그룹 버프가 없을때 표시합니다.",
		["Class Filter"] = "직업 필터",
		["Show status for the selected classes."] = "선택된 직업의 상태만을 표시합니다.",

		["Add Buff Group"] = "그룹 버프 추가",
		["Adds a new buff group to the status module"] = "상태 모듈에 새로운 그룹 버프를 추가합니다.",
		["Delete Buff Group"] = "그룹 버프 삭제",
		["Deletes an existing buff group from the status module"] = "상태 모듈의 기존 그룹 버프를 삭제합니다.",
		["Remove %s from the menu"] = "메뉴에서 %s|1을;를; 제거합니다.",
	}
end)

GridStatusMissingBuffs = GridStatus:NewModule("GridStatusMissingBuffs")
GridStatusMissingBuffs.menuName = L["Missing Buffs"]

local function compactSpellName(spell)
	return string.gsub(spell, L["[^%a]"], "")
end

local function statusForSpell(spell)
	return "buffGroup_"..compactSpellName(spell)
end

GridStatusMissingBuffs.defaultDB = {
	debug = false, 
	buffGroup_MarkoftheWild = {
		text = BS["Mark of the Wild"],
		desc = L["Buff Group: "]..BS["Mark of the Wild"],
		icon = "MarkoftheWild",
		buffs = {
			MarkoftheWild = BS["Mark of the Wild"],
			GiftoftheWild = BS["Gift of the Wild"],
		},
		classFilter = {},
		combat = false,
		dead = false,
		offline = false,
		enable = false,
		color = { r = 0, g = 0, b = 1, a = 1 },
		priority = 99,
		range = false,
	},
	buffGroup_Fortitude = {
		text = BS["Power Word: Fortitude"],
		desc = L["Buff Group: "]..BS["Power Word: Fortitude"],
		icon = "PrayerofForitude",
		buffs = {
			PowerWordForitude = BS["Power Word: Fortitude"],
			PrayerofForitude = BS["Prayer of Fortitude"],
		},
		classFilter = {},
		combat = false,
		dead = false,
		offline = false,
		enable = false,
		color = { r = 0, g = 0, b = 1, a = 1 },
		priority = 99,
		range = false,
	},
	buffGroup_DivineSpirit = {
		text = BS["Divine Spirit"],
		desc = L["Buff Group: "]..BS["Divine Spirit"],
		icon = "DivineSpirit",
		buffs = {
			DivineSpirit = BS["Divine Spirit"],
			PrayerofSpirit = BS["Prayer of Spirit"],
		},
		classFilter = {
			WARRIOR = false,
			ROGUE = false,
		},
		combat = false,
		dead = false,
		offline = false,
		enable = false,
		color = { r = 0, g = 0, b = 1, a = 1 },
		priority = 99,
		range = false,
	},
	buffGroup_ArcaneIntellect = {
		text = BS["Arcane Intellect"],
		desc = L["Buff Group: "]..BS["Arcane Intellect"],
		icon = "ArcaneBrilliance",
		buffs = {
			ArcaneIntellect = BS["Arcane Intellect"],
			ArcaneBrilliance = BS["Arcane Brilliance"],
		},
		classFilter = {
			WARRIOR = false,
			ROGUE = false,
		},
		combat = false,
		dead = false,
		offline = false,
		enable = false,
		color = { r = 0, g = 0, b = 1, a = 1 },
		priority = 99,
		range = false,
	},
	buffGroup_BlessingofKings = {
		text = BS["Blessing of Kings"],
		desc = L["Buff Group: "]..BS["Blessing of Kings"],
		icon = "BlessingofKings",
		buffs = {
			BlessingofKings = BS["Blessing of Kings"],
			GreaterBlessingofKings = BS["Greater Blessing of Kings"],
		},
		classFilter = {},
		combat = false,
		dead = false,
		offline = false,
		enable = false,
		color = { r = 0, g = 0, b = 1, a = 1 },
		priority = 99,
		range = false,
	},
	buffGroup_BlessingofMight = {
		text = BS["Blessing of Might"],
		desc = L["Buff Group: "]..BS["Blessing of Might"],
		icon = "BlessingofMight",
		buffs = {
			BlessingofMight = BS["Blessing of Might"],
			GreaterBlessingofMight = BS["Greater Blessing of Might"],
		},
		classFilter = {
			DRUID = false,
			PRIEST = false,
			SHAMAN = false,
			MAGE = false,
			WARLOCK = false,
			PALADIN = false,
		},
		combat = false,
		dead = false,
		offline = false,
		enable = false,
		color = { r = 0, g = 0, b = 1, a = 1 },
		priority = 99,
		range = false,
	},
	buffGroup_BlessingofSalvation = {
		text = BS["Blessing of Salvation"],
		desc = L["Buff Group: "]..BS["Blessing of Salvation"],
		icon = "BlessingofSalvation",
		buffs = {
			BlessingofSalvation = BS["Blessing of Salvation"],
			GreaterBlessingofSalvation = BS["Greater Blessing of Salvation"],
		},
		classFilter = {
			WARRIOR = false,
		},
		combat = false,
		dead = false,
		offline = false,
		enable = false,
		color = { r = 0, g = 0, b = 1, a = 1 },
		priority = 99,
		range = false,
	},
	buffGroup_BlessingofWisdom = {
		text = BS["Blessing of Wisdom"],
		desc = L["Buff Group: "]..BS["Blessing of Wisdom"],
		icon = "BlessingofWisdom",
		buffs = {
			BlessingofWisdom = BS["Blessing of Wisdom"],
			GreaterBlessingofWisdom = BS["Greater Blessing of Wisdom"],
		},
		classFilter = {
			WARRIOR = false,
			ROGUE = false,
		},
		combat = false,
		dead = false,
		offline = false,
		enable = false,
		color = { r = 0, g = 0, b = 1, a = 1 },
		priority = 99,
		range = false,
	},
	buffGroup_BlessingofLight = {
		text = BS["Blessing of Light"],
		desc = L["Buff Group: "]..BS["Blessing of Light"],
		icon = "BlessingofLight",
		buffs = {
			BlessingofLight = BS["Blessing of Light"],
			GreaterBlessingofLight = BS["Greater Blessing of Light"],
		},
		classFilter = {},
		combat = false,
		dead = false,
		offline = false,
		enable = false,
		color = { r = 0, g = 0, b = 1, a = 1 },
		priority = 99,
		range = false,
	},
	buffGroup_BlessingofSanctuary = {
		text = BS["Blessing of Sanctuary"],
		desc = L["Buff Group: "]..BS["Blessing of Sanctuary"],
		icon = "BlessingofSanctuary",
		buffs = {
			BlessingofSanctuary = BS["Blessing of Sanctuary"],
			GreaterBlessingofSanctuary = BS["Greater Blessing of Sanctuary"],
		},
		classFilter = {},
		combat = false,
		dead = false,
		offline = false,
		enable = false,
		color = { r = 0, g = 0, b = 1, a = 1 },
		priority = 99,
		range = false,
	},
}
 
function GridStatusMissingBuffs:OnInitialize()
	self.super.OnInitialize(self)

	self:RegisterStatuses()
end

function GridStatusMissingBuffs:OnEnable()
	self.debugging = self.db.profile.debug
	self:CreateAddRemoveOptions()
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("PLAYER_LEAVING_WORLD")
	self:RegisterEvent("PLAYER_REGEN_ENABLED", "UpdateAllUnits")
	self:RegisterEvent("PLAYER_REGEN_DISABLED", "UpdateAllUnits")
	self:RegisterEvent("PLAYER_ALIVE", "UpdateAllUnits")
	self:RegisterEvent("PLAYER_DEAD", "UpdateAllUnits")
	self:RegisterEvent("PLAYER_UNGHOST", "UpdateAllUnits")
	self:RegisterEvent("SpecialEvents_UnitBuffGained", "UpdateUnit")
	self:RegisterEvent("SpecialEvents_UnitBuffLost", "UpdateUnit")
	self:RegisterEvent("Grid_UnitJoined")
	self:CreateAddRemoveOptions()
end

function GridStatusMissingBuffs:PLAYER_LEAVING_WORLD()
	self:UnregisterEvent("SpecialEvents_UnitBuffGained")
	self:UnregisterEvent("SpecialEvents_UnitBuffLost")
end

function GridStatusMissingBuffs:PLAYER_ENTERING_WORLD()
	self:UpdateAllUnits()
	self:RegisterEvent("SpecialEvents_UnitBuffGained", "UpdateUnit")
	self:RegisterEvent("SpecialEvents_UnitBuffLost", "UpdateUnit")
end

function GridStatusMissingBuffs:Grid_UnitJoined(name, unitid)
	self:Debug("Joined",name)
	self:UpdateUnit(unitid)
end

function GridStatusMissingBuffs:RegisterStatuses()
	local status, settings, desc

	for status, settings in self:ConfiguredStatusIterator() do
		desc = settings.desc or settings.text
		if not settings.classFilter then
			self.db.profile[status].classFilter = {}
		end
		self:Debug("registering", status, desc)
		self:RegisterStatus(status, desc, self:GetBuffGroupOptions(status))
	end
end

function GridStatusMissingBuffs:UnregisterStatuses()
	local status, moduleName, desc
	for status, moduleName, desc in self.core:RegisteredStatusIterator() do
		if moduleName == self.name then
			self:Debug("unregistering", status, desc)
			self:UnregisterStatus(status)
			self.options.args[status] = nil
		end
	end
end

function GridStatusMissingBuffs:ConfiguredStatusIterator()
	local profile = self.db.profile
	local status

	return function ()
		status = next(profile, status)

		-- skip any non-table entries
		while status ~= nil and type(profile[status]) ~= "table" do
			status = next(profile, status)
		end

		if status == nil then
			return nil
		end

		return status, profile[status]
	end
end

function GridStatusMissingBuffs:GetBuffGroupOptions(status)
	local classFilterArgs = {}

	local classes = {
		WARRIOR = BC["Warrior"],
		PRIEST = BC["Priest"],
		DRUID = BC["Druid"],
		PALADIN = BC["Paladin"],
		SHAMAN = BC["Shaman"],
		MAGE = BC["Mage"],
		WARLOCK = BC["Warlock"],
		HUNTER = BC["Hunter"],
		ROGUE = BC["Rogue"],
	}

	for class,name in pairs(classes) do
		local class,name = class,name
		classFilterArgs[class] = {
			type = "toggle",
			name = name,
			desc = string.format(L["Show on %s."], name),
			get = function ()
				return self.db.profile[status].classFilter[class] ~= false
			end,
			set = function (v)
				 self.db.profile[status].classFilter[class] = v
			end,
		}
	end

	return {
		add_buff = {
			type = "text",
			name = L["Add Buff"],
			desc = L["Adds a new buff to the buff group"],
			get = false,
			usage = L["<buff name>"],
			set = function(v) self:AddBuff(status, v) end,
			order = 110.8
		},
		delete_buff = {
			type = "group",
			name = L["Delete Buff"],
			desc = L["Deletes an existing buff from the buff group"],
			disable = function() return next(self.db.profile[status].buffs) == nil end,
			args = {},
			order = 110.9
		},
		buff_icon = {
			type = 'text',
			name = L["Buff Icon"],
			desc = L["The icon to appear when no buffs in the buff group are active."],
			usage =  L["<setName>"],
			validate = self.db.profile[status].buffs,
			disable = function() return next(self.db.profile[status].buffs) == nil end,
			get = function()
				return self.db.profile[status].icon
			end,
			set = function(v)
				self.db.profile[status].icon = v
			end,
		},
		show = {
			type = "group",
			name =  L["Show"],
			desc =  L["Configure when to show the buff group"],
			args = {
				dead = {
					type = "toggle",
					name =  L["When Dead"],
					desc =  L["Show missing buff group for dead players"],
					get = function()
						return self.db.profile[status].dead
					end,
					set = function(v)
						self.db.profile[status].dead = not self.db.profile[status].dead
					end,
				},
				offline = {
					type = "toggle",
					name =  L["When Offline"],
					desc =  L["Show missing buff group for offline players"],
					get = function()
						return self.db.profile[status].offline
					end,
					set = function(v)
						self.db.profile[status].offline = not self.db.profile[status].offline
					end,
				},					
				combat = {
					type = "toggle",
					name =  L["In Combat"],
					desc =  L["Show missing buff group when in combat"],
					get = function()
						return self.db.profile[status].combat
					end,
					set = function()
						self.db.profile[status].combat = not self.db.profile[status].combat
					end,
				},
			}
		},
		classFilter = {
			type = "group",
			name =  L["Class Filter"],
			desc =  L["Show status for the selected classes."],
			args = classFilterArgs,
			order = 111
		},
	}
end

function GridStatusMissingBuffs:Reset()
	self.super.Reset(self)

	self:UnregisterStatuses()
	self:RegisterStatuses()
	self:CreateAddRemoveOptions()
end

function GridStatusMissingBuffs:CreateAddRemoveOptions()
	local status, settings

	self.options.args["add_buff_group"] = {
		type = "text",
		name =  L["Add Buff Group"],
		desc =  L["Adds a new buff group to the status module"],
		get = false,
		usage =  L["<buff name>"],
		set = function(v) self:AddBuffGroup(v) end,
		order = 201
	}
	self.options.args["delete_buff_group"] = {
		type = "group",
		name =  L["Delete Buff Group"],
		desc =  L["Deletes an existing buff group from the status module"],
		disable = function() return next(self.db.profile[status]) == nil end,
		args = {},
		order = 202
	}

	for status, settings in self:ConfiguredStatusIterator() do
		local status, settings = status, settings
		local buffName = (settings.desc or settings.text)
		self.options.args["delete_buff_group"].args[status] = {
			type = "execute",
			name = buffName,
			desc = string.format(L["Remove %s from the menu"], buffName),
			func = function() return self:DeleteBuffGroup(status) end
		}

		self.options.args["delete_buff_group"].args[status] = {
			type = "execute",
			name = buffName,
			desc = string.format(L["Remove %s from the menu"], buffName),
			func = function() return self:DeleteBuffGroup(status) end
		}
		local buffId, buffName
		for buffId, buffName in pairs(settings.buffs) do
			local buffId, buffName = buffId, buffName
			self.options.args[status].args["delete_buff"].args[buffId] = {
				type = "execute",
				name = buffName,
				desc = string.format(L["Remove %s from the menu"], buffName),
				func = function() return self:DeleteBuff(status, buffId) end
			}
		end
	end
end

function GridStatusMissingBuffs:AddBuffGroup(name)
	local status = statusForSpell(name)
	local desc = "Buff Group: "..name
	local buff = BS:HasReverseTranslation(name) and name
	local compactName = compactSpellName(name)

	self.db.profile[status] = {
		text = name,
		desc = desc,
		enable = true,
		buffs = {
			[compactName] = buff,
		},
		icon = compactName,
		priority = 90,
		range = false,
		color = { r = 0, g = 0, b = 1, a = 1 },
		classFilter = {},
		combat = false,
		dead = false,
		offline = false,
	}

	self:RegisterStatus(status, desc, self:GetBuffGroupOptions(status))
	self:CreateAddRemoveOptions()
end

function GridStatusMissingBuffs:DeleteBuffGroup(status)
	self:UnregisterStatus(status)
	self.options.args[status] = nil
	self.options.args["delete_buff_group"].args[status] = nil
	self.db.profile[status] = nil
	self:CreateAddRemoveOptions()
end

function GridStatusMissingBuffs:AddBuff(status, name)
	local buffId = statusForSpell(name)

	self.db.profile[status].buffs[buffId] = name
end

function GridStatusMissingBuffs:DeleteBuff(status, buffId)
	self.options.args[status].args["delete_buff"].args[buffId] = nil
	self.db.profile[status].buffs[buffId] = nil
end

function GridStatusMissingBuffs:UpdateAllUnits()
	self:Debug("updating all units")
	for u in RL:IterateRoster() do
		self:UpdateUnit(u.unitid)
	end
end

function GridStatusMissingBuffs:UpdateUnit(unitid)
	self:Debug("Updating Unit",UnitName(unitid))

	for status in self:ConfiguredStatusIterator() do
		self:ShowMissingBuffs(unitid, status)
	end
end

function GridStatusMissingBuffs:ShowMissingBuffs(unit, status)
	local name = UnitName(unit)
	local settings = self.db.profile[status]

	if not settings.enable or
	   (not settings.combat and (UnitAffectingCombat("player") or UnitAffectingCombat(unit))) or
	   (not settings.dead and UnitIsDeadOrGhost(unit)) or
	   (not settings.offline and not UnitIsConnected(unit))
	then
		return self.core:SendStatusLost(name, status)
	end

	local _, class = UnitClass(unit)
	if settings.classFilter[class or ""] == false then
		return self.core:SendStatusLost(name, status)
	end

	for buffId, buff in pairs(settings.buffs) do
		if Aura:UnitHasBuff(unit, buff) then
			return self.core:SendStatusLost(name, status)
		end
	end
	
	local tex = BS:GetSpellIcon(settings.buffs[settings.icon or ""])
	self:Debug("gained",status)
	self.core:SendStatusGained(
		name,
		status,
		settings.priority,
		(settings.range and 40),
		settings.color,
		settings.text,
		nil,
		nil,
		tex)
end